const chalk = require("chalk");
const path = require("path");
const fs = require("fs");
const axios = require("axios");
const ora = require("ora");
const Utils = require("./utils");
const { MD5 } = require("./md5")
const { configFileName } = require("./const");

let translateResult = {}; // map 中英文对照表，result 翻译结果
let targetLocal = {}; // 需要翻译的中文包
let translateConfig = {}; // 项目配置文件
let targetLang = ""; // 要翻译的语言
const isObject = (obj) => Object.prototype.toString.call(obj) === '[object Object]'

/** 利用百度翻译，批量翻译 */
const baiduTranslate = (query, keyList) => {
	const appid = translateConfig.appId;
	const key = translateConfig.secret;
	const salt = new Date().getTime(); // 取当前时间作为随机数
	const str1 = appid + query + salt + key; // 秘钥
	const sign = MD5(str1); // md5加密

	return new Promise(function (resolve, reject) {
		axios({
			url: "http://api.fanyi.baidu.com/api/trans/vip/translate",
			params: {
				q: query,
				appid: appid,
				salt: salt,
				from: "zh",
				to: targetLang, // 翻译目标语言
				sign: sign,
			},
			method: "GET",
		})
			.then((res) => {
				if (res.data) {
					const data = res.data;
					if (data.error_code) {
						console.log(chalk.red(` ❌ ${query} 百度翻译未成功翻译!(error_code:${data.error_code})\n`));
						console.log(data);
						console.log("\n");
					} else {
						const trans_result = data.trans_result;
						trans_result.forEach((item, index) => {
							let dst = item.dst
							if (targetLang === "en") {
								dst = Utils.replaceStr(item.src, item.dst);
							}
							translateResult[keyList[index]] = dst;
						});
					}
					// 延时返回
					setTimeout(() => {
						if (data.error_code) {
							resolve(false);
						} else {
							resolve(true);
						}
					}, 1500);
				}
			})
			.catch((err) => {
				reject(err);
				console.log(chalk.red(" ❌ 百度翻译出现网络错误\n"));
			});
	});
};
// 获取语言包的总map
const getTranslateMap = (object, keyList = []) => {
	const langMap = {};
	for (const key in object) {
		const keyPath = keyList.concat([key])
		if (isObject(object[key])) {
			const childrenResult = getTranslateMap(object[key], keyPath)
			Object.assign(langMap, childrenResult);
		} else {
			langMap[keyPath.join("_")] = object[key]
		}
	}
	return langMap
}

/**
 * @description: 翻译
 * @author: - wsz -
 * @param {*} code 翻译的语言
 * @param {*} filename 进行翻译的目标文件
 */
module.exports = async function (code, filename) {
	targetLang = code;
	let processFile = "";
	const spinner = ora(chalk.blue(" ♻️ 开始翻译，请稍后......"));

	spinner.start();
	// 检测有无初始化配置文件
	if (fs.existsSync(configFileName)) {
		processFile = path.join(process.cwd(), configFileName);
		translateConfig = require(processFile);
	} else {
		spinner.fail(chalk.red(`\n ❌ ${configFileName}文件不存在，请先初始化项目\n`));
		return;
	}
	let file = translateConfig.file;

	let targetFilePath = path.join(translateConfig.dir, targetLang);

	if (filename) {
		file = filename.indexOf(".") > -1 ? filename : `${filename}.js`;
		const pointLocation = file.indexOf(".");
		const toCodeName = file.substr(0, pointLocation);
		targetFilePath = path.join(translateConfig.dir, targetLang + "-" + toCodeName);
	}
	// 获取源语言包
	let sourceLocal = {};
	let sourceLocalFileData = {};
	if (fs.existsSync(translateConfig.dir + file)) {
		processFile = path.join(process.cwd(), translateConfig.dir + file);
		sourceLocalFileData = require(processFile);
		// 判断是否传入了localkey 并检测
		if (sourceLocalFileData.hasOwnProperty("local")) {
			sourceLocal = sourceLocalFileData.local;
		} else {
			spinner.fail(chalk.red("\n ❌ local数据 并不存在！"));
			return
		}

	} else {
		spinner.fail(chalk.red(`\n ❌ 翻译源文件不存在,%s\n`, translateConfig.dir + file));
		return;
	}
	// 获取目标语言包
	let targetLocalFileData = {};
	if (fs.existsSync(targetFilePath + ".js")) {
		processFile = path.join(process.cwd(), targetFilePath + ".js");
		targetLocalFileData = require(processFile);
		targetLocal = targetLocalFileData.local || {};
		translateResult = targetLocal;
	}

	let query = ""; // 需要翻译的文本
	let keyList = [];
	// 通过对比中文json和英文josn，取出未翻译的中文json
	const noTranslationList = Object.keys(sourceLocal).filter((targetKey) => !targetLocal.hasOwnProperty(targetKey));
	let translateState = true; // 翻译成功与否的状态
	console.log(chalk.yellow(`\n进行翻译处理任务，请稍后(共${noTranslationList.length}个)`))
	for (let i = 0; i < noTranslationList.length; i++) {
		const key = noTranslationList[i];
		if (translateState) {
			// 大于 1500个字符。进行翻译 减少接口访问频率
			if (String(query).length >= 1500) {
				console.log(chalk.yellow(`\n 🌍 正在翻译,本轮共${keyList.length}个,进度(${i + 1}/${noTranslationList.length})......\n`))
				translateState = await baiduTranslate(query, keyList);
				query = sourceLocal[key];
				keyList = [key];
			} else {
				// 拼接查询文本
				query += "\n" + sourceLocal[key];
				keyList.push(key);
				if (i === noTranslationList.length - 1) {
					translateState = await baiduTranslate(query, keyList);
				}
			}
		}
	}
	if (translateState) {
		const file = path.join(process.cwd(), targetFilePath + ".js");
		const writeFileData = Object.assign({}, targetLocalFileData, {
			local: translateResult
		})
		fs.writeFileSync(file, `module.exports = ${JSON.stringify(writeFileData, null, "\t")};`, "utf8");
		spinner.succeed(chalk.green(" ✅ 翻译完成"));
	} else {
		spinner.fail(chalk.red("\n ❌ 翻译错误，请勿输入不正确的code"));
	}
};
